/* css for the whole site */

body {
	font-family:Arial, Helvetica, sans-serif !important;
	background-color:#ffffff;
}

/* Header and footer */

div.header {
	font-size:20px;
	color:#0000A0;
}

div.footer {
	font-size:10px;
	color:#3232d2;
	position: absolute float;
	bottom: 0px;
}

/* menu for the system*/

div.menu_system {
/*	width:100%;
	height:63px;*/
}

div.menu_system ul {
	list-style-type:none;
	margin:0;
	padding:0;
	position: absolute;
	top: 0px;
	right: 0px;
}

div.menu_system li {
	float:left;
}

div.menu_system a {
	display:block;
	width:86px;
}

div.menu_system a:link,div.menu_system a:visited {
	font-family:Arial;
	font-size:10px;
	color:#FFFFFF;
	background-color:#0000A0;
	text-align:center;
	padding:4px;
	text-decoration:none;
}

div.menu_system a:hover,div.menu_system a:active {
	background-color:#3232D2;
}


/* error message */
div.error {
	padding:2px 4px;
	margin:0px;
	border:solid 1px #FBD3C6;
	background:#FDE4E1;
	color:#CB4721;
	font-size:14px;
	font-weight:bold;
	text-align:center;
}

div.success {
	padding:2px 4px;
	margin:0px;
	border:solid 1px #C0F0B9;
	background:#D5FFC6;
	color:#48A41C; 
	font-family:Arial, Helvetica, sans-serif; font-size:14px; 
	font-weight:bold;
	text-align:center;
}

/* Required tags */
div.required {
	color:#ff0000;
	font-weight:bold;
}

/* table definitions */
table {
	border-collapse:collapse;
}

table, th, td {
	border: 1px solid #3232d2;
}

td {
	font-size:10px;
}

th {
	font-size:11px;
	font-weight:bold;
}

/* Pager */
div.pager
	{
		font-size:30px;
		color:#ffff00;
	}
/* Titles */
div.title {
	font-size: 30px;
	font-weight:bold;
	color:#ff0000;
}

div.maintitle {
	font-size: 20px;
	font-weight: bold;
	background-color:#00ffff;
	position:absolute;
	left:50%;
	top:0;
}
/* Experimenting with making an iPhone version of the code */

@media screen and (max-device-width: 480px){
    div.menu_system a:link,div.menu_system a:visited 
    	{
		font-size:10px;
	}
	div.menu_system a 
	{
		width:60px;
	}
}

/* Experimenting with an iPad version of the style sheet  as well */
@media only screen and (min-device-width: 768px) and (max-device-width: 1024px) {

}
